==> dassadmin_2.59.tar.gz: dassadmin/src/de/dassit/ldap/Connection.java <==
/*
 * Copyright 2010 Sebastian Lederer <slederer@insignificance.de>
 */

package de.dassit.ldap;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.CommunicationException;
import javax.naming.Context;
import javax.naming.InvalidNameException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.LdapName;

public class Connection {
	private static final Logger log = Logger.getLogger(Connection.class
			.getName());

	static protected volatile Connection defaultConnection;

	protected DirContext ctx;

	protected boolean isConnected;

	protected String bindDn = "uid=admin,ou=system";

	protected String bindPassword = "password";

	protected String baseDn = "dc=example,dc=com";

	protected String ldapUri = "ldap://localhost";

	protected ChangeLogger logger = new NullLogger();

	public static Connection getDefaultConnection() {
		if (defaultConnection != null)
			return defaultConnection;
		synchronized (Connection.class) {
			if (defaultConnection == null)
				defaultConnection = new Connection();
		}
		defaultConnection.validate();
		return defaultConnection;
	}

	public static void setDefaultConnection(Connection c) {
		defaultConnection = c;
	}

	public void setChangeLogger(ChangeLogger l) {
		logger = l;
	}

	public void validate() {
		if (!isConnected)
			connect();
	}

	public String getLdapUri() {
		return ldapUri;
	}

	public void setLdapUri(String l) {
		this.ldapUri = l;
	}

	public String getBaseDn() {
		return baseDn;
	}

	public void setBaseDn(String baseDn) {
		this.baseDn = baseDn;
	}

	public String getBindDn() {
		return bindDn;
	}

	public void setBindDn(String bindDn) {
		this.bindDn = bindDn;
	}

	public String getBindPassword() {
		return bindPassword;
	}

	public void setBindPassword(String bindPassword) {
		this.bindPassword = bindPassword;
	}

	public void addEntry(Entry e, String suffix) throws NamingException {
		String s;

		if (suffix == null || suffix.equals(""))
			s = baseDn;
		else
			s = suffix + "," + baseDn;

		String dn = e.getNamingAttribute() + "=" + e.getName() + "," + s;
		ctx.createSubcontext(dn, e.getAttributes());
		logger.create(dn);
	}

	public void deleteEntry(Entry e) throws NamingException {
		String dn = e.getRDN() + "," + baseDn;
		ctx.destroySubcontext(dn);
		logger.delete(dn);
	}

	public void renameEntry(Entry e, String newFullName) throws NamingException {
		String oldDn, newDn;

		oldDn = e.getRDN() + "," + baseDn;
		newDn = newFullName + "," + baseDn;
		ctx.rename(oldDn, newDn);
		e.setDN(newDn);
		logger.rename(oldDn, newDn);
	}

	public void modifyEntry(Entry e, ModificationItem[] mods)
			throws NamingException {
		String dn = e.getRDN() + "," + baseDn;
		ctx.modifyAttributes(dn, mods);
		logger.update(dn);
	}

	public void searchWithRetry(String dn, String filter, SearchControls sc,
			EntryList<? extends Entry> results) {
		@SuppressWarnings("rawtypes")
		NamingEnumeration ne = null;

		log.info("search " + dn + " " + filter);
		try {
			try {
				ne = ctx.search(dn, filter, sc);
			} catch (CommunicationException cex) {
				log.info("Connection failed (" + cex.getMessage()
						+ "), reconnecting");
				isConnected = false;
				// reconnect and retry
				validate();
				ne = ctx.search(dn, filter, sc);
			}
			while (ne.hasMore()) {
				SearchResult sr = (SearchResult) ne.next();
				results.addInstance(sr);
			}
		} catch (NamingException nex) {
			log.warning("Error searching for " + filter + " at " + dn
					+ ": " + nex.getMessage());
		}
	}

	public static String getRelativeDn(String dn, String baseDn) {
		String relDn = dn.substring(0, dn.length() - baseDn.length()).trim();
		if (relDn.endsWith(","))
			relDn = relDn.substring(0, relDn.length() - 1);

		return relDn;
	}

	public void getByDN(String dn, EntryList<? extends Entry> results) {
		SearchControls sc = new SearchControls();
		sc.setSearchScope(SearchControls.OBJECT_SCOPE);

		searchWithRetry(dn, "objectClass=*", sc, results);

		String relativeDn = getRelativeDn(dn, baseDn);

		LdapName n;
		try {
			n = new LdapName(dn);
		} catch (InvalidNameException e1) {
			e1.printStackTrace();
			return;
		}

		String fullName = n.getRdn(n.size() - 1).toString();
		Iterator<? extends Entry> i = results.getEntries().iterator();
		while (i.hasNext()) {
			Entry e = (Entry) i.next();
			final String container = relativeDn
					.substring(fullName.length() + 1);
			e.setContainer(container);
			e.setFullName(fullName);
		}
	}

	public void search(String filter, String relativeDn,
			EntryList<? extends Entry> results) {
		String dn;

		if (relativeDn.equals(""))
			dn = relativeDn;
		else
			dn = relativeDn + ",";

		SearchControls sc = new SearchControls();
		sc.setSearchScope(SearchControls.SUBTREE_SCOPE);

		searchWithRetry(dn + baseDn, filter, sc, results);

		Iterator<? extends Entry> i = results.getEntries().iterator();
		while (i.hasNext()) {
			Entry e = (Entry) i.next();
			e.setContainer(relativeDn);
		}

	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public void connect() {
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY,
				"com.sun.jndi.ldap.LdapCtxFactory");
		env.put(Context.PROVIDER_URL, ldapUri);
		env.put(Context.SECURITY_AUTHENTICATION, "simple");
		env.put(Context.SECURITY_PRINCIPAL, bindDn); // specify the username
		env.put(Context.SECURITY_CREDENTIALS, bindPassword);
		env.put(Context.REFERRAL, "throw");

		try {
			ctx = new InitialDirContext(env);
			isConnected = true;

		} catch (NamingException e) {
			log.log(Level.SEVERE, "Connecting to LDAP", e);
		}
	}

	public void close() {
		try {
			ctx.close();
		} catch (NamingException e) {
			log.log(Level.WARNING, "Closing LDAP connection",e);
		}
		isConnected = false;
	}

}

==> dassadmin_2.62.tar.gz: dassadmin/src/de/dassit/ldap/Connection.java <==
/*
 * Copyright 2010 Sebastian Lederer <slederer@insignificance.de>
 */

package de.dassit.ldap;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.CommunicationException;
import javax.naming.Context;
import javax.naming.InvalidNameException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;
import javax.naming.ldap.LdapName;
import javax.naming.ldap.StartTlsRequest;
import javax.naming.ldap.StartTlsResponse;

public class Connection {
	private static final Logger log = Logger.getLogger(Connection.class
			.getName());

	static protected volatile Connection defaultConnection;

	protected LdapContext ctx;

	protected boolean isConnected;

	protected String bindDn = "uid=admin,ou=system";

	protected String bindPassword = "password";

	protected String baseDn = "dc=example,dc=com";

	protected String ldapUri = "ldap://localhost";

	protected ChangeLogger logger = new NullLogger();

	private boolean useStartTLS;

	public static Connection getDefaultConnection() {
		if (defaultConnection != null)
			return defaultConnection;
		synchronized (Connection.class) {
			if (defaultConnection == null)
				defaultConnection = new Connection();
		}
		// defaultConnection.validate();
		return defaultConnection;
	}

	public static void setDefaultConnection(Connection c) {
		defaultConnection = c;
	}

	public boolean isConnected() {
		return isConnected;
	}

	public void setChangeLogger(ChangeLogger l) {
		logger = l;
	}

	public void validate() throws IOException {
		if (!isConnected)
			connect();
	}

	public String getLdapUri() {
		return ldapUri;
	}

	public void setLdapUri(String l) {
		this.ldapUri = l;
	}

	public String getBaseDn() {
		return baseDn;
	}

	public void setBaseDn(String baseDn) {
		this.baseDn = baseDn;
	}

	public String getBindDn() {
		return bindDn;
	}

	public void setBindDn(String bindDn) {
		this.bindDn = bindDn;
	}

	public String getBindPassword() {
		return bindPassword;
	}

	public void setBindPassword(String bindPassword) {
		this.bindPassword = bindPassword;
	}

	/**
	 * @return the useStartTLS
	 */
	public boolean isUseStartTLS() {
		return useStartTLS;
	}

	/**
	 * @param useStartTLS
	 *            the useStartTLS to set
	 */
	public void setUseStartTLS(boolean useStartTLS) {
		this.useStartTLS = useStartTLS;
	}

	public void configure(Properties conf) {
		setLdapUri(conf.getProperty("url", "ldap://localhost"));
		setBaseDn(conf.getProperty("basedn"));
		setBindDn(conf.getProperty("binddn"));
		setBindPassword(conf.getProperty("bindpw"));

		String tls = conf.getProperty("starttls");

		if (tls == null)
			return;

		if (tls.equalsIgnoreCase("true") || tls.equalsIgnoreCase("yes"))
			setUseStartTLS(true);
	}

	public void addEntry(Entry e, String suffix) throws NamingException {
		String s;

		if (suffix == null || suffix.equals(""))
			s = baseDn;
		else
			s = suffix + "," + baseDn;

		String dn = e.getNamingAttribute() + "=" + e.getName() + "," + s;
		ctx.createSubcontext(dn, e.getAttributes());
		logger.create(dn);
	}

	public void deleteEntry(Entry e) throws NamingException {
		String dn = e.getRDN() + "," + baseDn;
		ctx.destroySubcontext(dn);
		logger.delete(dn);
	}

	public void renameEntry(Entry e, String newFullName) throws NamingException {
		String oldDn, newDn;

		oldDn = e.getRDN() + "," + baseDn;
		newDn = newFullName + "," + baseDn;
		ctx.rename(oldDn, newDn);
		e.setDN(newDn);
		logger.rename(oldDn, newDn);
	}

	public void modifyEntry(Entry e, ModificationItem[] mods)
			throws NamingException {
		String dn = e.getRDN() + "," + baseDn;
		ctx.modifyAttributes(dn, mods);
		logger.update(dn);
	}

	public void searchWithRetry(String dn, String filter, SearchControls sc,
			EntryList<? extends Entry> results) {
		@SuppressWarnings("rawtypes")
		NamingEnumeration ne = null;

		log.info("search " + dn + " " + filter);
		try {
			try {
				ne = ctx.search(dn, filter, sc);
			} catch (CommunicationException cex) {
				log.info("Connection failed (" + cex.getMessage()
						+ "), reconnecting");
				isConnected = false;
				// reconnect and retry
				try {
					validate();
				} catch (IOException e) {
					log.info("Connection failed (" + e.getMessage()
							+ "), stalled");
					return;
				}
				ne = ctx.search(dn, filter, sc);
			}
			while (ne.hasMore()) {
				SearchResult sr = (SearchResult) ne.next();
				results.addInstance(sr);
			}
		} catch (NamingException nex) {
			log.warning("Error searching for " + filter + " at " + dn + ": "
					+ nex.getMessage());
		}
	}

	public static String getRelativeDn(String dn, String baseDn) {
		String relDn = dn.substring(0, dn.length() - baseDn.length()).trim();
		if (relDn.endsWith(","))
			relDn = relDn.substring(0, relDn.length() - 1);

		return relDn;
	}

	public void getByDN(String dn, EntryList<? extends Entry> results) {
		SearchControls sc = new SearchControls();
		sc.setSearchScope(SearchControls.OBJECT_SCOPE);

		searchWithRetry(dn, "objectClass=*", sc, results);

		String relativeDn = getRelativeDn(dn, baseDn);

		LdapName n;
		try {
			n = new LdapName(dn);
		} catch (InvalidNameException e1) {
			e1.printStackTrace();
			return;
		}

		String fullName = n.getRdn(n.size() - 1).toString();
		Iterator<? extends Entry> i = results.getEntries().iterator();
		while (i.hasNext()) {
			Entry e = (Entry) i.next();
			final String container = relativeDn
					.substring(fullName.length() + 1);
			e.setContainer(container);
			e.setFullName(fullName);
		}
	}

	public void search(String filter, String relativeDn,
			EntryList<? extends Entry> results) {
		String dn;

		if (relativeDn.equals(""))
			dn = relativeDn;
		else
			dn = relativeDn + ",";

		SearchControls sc = new SearchControls();
		sc.setSearchScope(SearchControls.SUBTREE_SCOPE);

		searchWithRetry(dn + baseDn, filter, sc, results);

		Iterator<? extends Entry> i = results.getEntries().iterator();
		while (i.hasNext()) {
			Entry e = (Entry) i.next();
			e.setContainer(relativeDn);
		}

	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public void connect() throws IOException {
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY,
				"com.sun.jndi.ldap.LdapCtxFactory");
		env.put(Context.PROVIDER_URL, ldapUri);
		env.put(Context.SECURITY_AUTHENTICATION, "simple");
		env.put(Context.SECURITY_PRINCIPAL, bindDn); // specify the username
		env.put(Context.SECURITY_CREDENTIALS, bindPassword);
		env.put(Context.REFERRAL, "throw");

		try {
			ctx = new InitialLdapContext(env, null);

			if (useStartTLS) {
				// Start TLS
				StartTlsResponse tls = (StartTlsResponse) ctx
						.extendedOperation(new StartTlsRequest());
				tls.negotiate();
			}
			isConnected = true;

		} catch (NamingException e) {
			log.log(Level.SEVERE, "Connecting to LDAP", e);
		}
	}

	public void close() {
		try {
			ctx.close();
		} catch (NamingException e) {
			log.log(Level.WARNING, "Closing LDAP connection", e);
		}
		isConnected = false;
	}

}
